@@ -1,8 +1,8 @@
 /*
  * cutils.c module.
- * 
+ *
  * Miscellaneous functions to speed up the IMDbPY package.
- * 
+ *
  * Contents:
  * - pyratcliff():
  *   Function that implements the Ratcliff-Obershelp comparison
@@ -20,23 +20,23 @@
  * - pysoundex():
  *   Return a soundex code string, for the given string.
  *
- * Copyright 2004-2006 Davide Alberani <da@erlug.linux.it>
+ * Copyright 2004-2008 Davide Alberani <da@erlug.linux.it>
  * Released under the GPL license.
- * 
+ *
  * NOTE: The Ratcliff-Obershelp part was heavily based on code from the
  * "simil" Python module.
  * The "simil" module is copyright of Luca Montecchiani <cbm64 _at_ inwind.it>
  * and can be found here: http://spazioinwind.libero.it/montecchiani/
  * It was released under the GPL license; original comments are leaved
  * below.
- * 
+ *
  */
 
 
 /*========== Ratcliff-Obershelp ==========*/
 /*****************************************************************************
  *
- * Stolen code from : 
+ * Stolen code from :
  *
  * [Python-Dev] Why is soundex marked obsolete?
  * by Eric S. Raymond [4]esr@thyrsus.com
@@ -67,8 +67,8 @@
 #define COMPARE             2.0
 #define STRING_MAXLENDIFFER 0.7
 
-/* As of 26 Mar 2006, the longest title is 280 chars. */
-#define MXLINELEN   512
+/* As of 05 Mar 2008, the longest title is ~600 chars. */
+#define MXLINELEN   1023
 #define FSEP        '|'
 
 #define RO_THRESHOLD 0.6
@@ -78,20 +78,20 @@
 
 /* List of articles.
    XXX: see comments about articles in the imdb.utils module. */
-#define ART_COUNT    45
+#define ART_COUNT    46
 char *articles[ART_COUNT] = {"the ", "la ", "a ", "die ", "der ", "le ", "el ",
-        "l'", "il ", "das ", "les ", "o ", "ein ", "i ", "un ", "los ", "de ",
-        "an ", "una ", "las ", "eine ", "den ", "gli ", "het ","os ", "lo ",
-        "az ", "det ","ha-", "een ", "ang ", "oi ", "ta ", "al-", "dem ",
-        "mga ", "uno ", "un'", "ett ", " ", "eines ", " ","els ", " ",
-        " "};
+        "l'", "il ", "das ", "les ", "i ", "o ", "ein ", "un ", "de ", "los ",
+        "an ", "una ", "las ", "eine ", "den ", "het ", "gli ", "lo ", "os ",
+        "ang ", "oi ", "az ", "een ", "ha-", "det ", "ta ", "al-",
+	"mga ", "un'", "uno ", "ett ", "dem ", "egy ", "els ", "eines ", " ",
+	" ", " ", " "};
 
 char *articlesNoSP[ART_COUNT] = {"the", "la", "a", "die", "der", "le", "el",
-        "l'", "il", "das", "les", "o", "ein", "i", "un", "los", "de",
-        "an", "una", "las", "eine", "den", "gli", "het", "os", "lo",
-        "az", "det","ha-", "een", "ang", "oi", "ta", "al-", "dem",
-        "mga", "uno", "un'", "ett", "", "eines", "", "els", "",
-        ""};
+        "l'", "il", "das", "les", "i", "o", "ein", "un", "de", "los",
+        "an", "una", "las", "eine", "den", "het", "gli", "lo", "os",
+        "ang", "oi", "az", "een", "ha-", "det", "ta", "al-",
+	"mga", "un'", "uno", "ett", "dem", "egy", "els", "eines", "",
+	"", "", ""};
 
 
 //*****************************************
@@ -100,7 +100,7 @@ char *articlesNoSP[ART_COUNT] = {"the", "la", "a", "die", "der", "le", "el",
 static float
 strings_check(char const *s, char const *t)
 {
-    float threshold;    // lenght difference 
+    float threshold;    // lenght difference
     int s_len = strlen(s);    // length of s
     int t_len = strlen(t);    // length of t
 
@@ -202,8 +202,8 @@ pyratcliff(PyObject *self, PyObject *pArgs)
     char *s1 = NULL;
     char *s2 = NULL;
     PyObject *discard = NULL;
-    char s1copy[MXLINELEN];
-    char s2copy[MXLINELEN];
+    char s1copy[MXLINELEN+1];
+    char s2copy[MXLINELEN+1];
 
     /* The optional PyObject parameter is here to be compatible
      * with the pure python implementation, which uses a
@@ -211,8 +211,8 @@ pyratcliff(PyObject *self, PyObject *pArgs)
     if (!PyArg_ParseTuple(pArgs, "ss|O", &s1, &s2, &discard))
         return NULL;
 
-    strcpy(s1copy, s1);
-    strcpy(s2copy, s2);
+    strncpy(s1copy, s1, MXLINELEN);
+    strncpy(s2copy, s2, MXLINELEN);
     /* Work on copies. */
     strtolower(s1copy);
     strtolower(s2copy);
@@ -238,23 +238,30 @@ search_name(PyObject *self, PyObject *pArgs, PyObject *pKwds)
     char *name3 = NULL;
     float ratio;
     FILE *keyFile;
-    char line[MXLINELEN];
-    char origLine[MXLINELEN];
-    char surname[MXLINELEN] = "";
-    char namesurname[MXLINELEN] = "";
+    char line[MXLINELEN+1];
+    char origLine[MXLINELEN+1];
+    char surname[MXLINELEN+1] = "";
+    char namesurname[MXLINELEN+1] = "";
     char *cp;
     char *key;
     short hasNS = 0;
     unsigned int nrResults = 0;
     static char *argnames[] = {"keyFile", "name1", "name2", "name3",
-                                "results", NULL};
+                                "results", "_scan_character", NULL};
+    PyObject *scanChar = NULL;
+    int isChar = 0;
     PyObject *result = PyList_New(0);
 
-    if (!PyArg_ParseTupleAndKeywords(pArgs, pKwds, "ss|ssi",
-                argnames, &keyFileName, &name1, &name2, &name3, &nrResults))
+    if (!PyArg_ParseTupleAndKeywords(pArgs, pKwds, "ss|ssiO",
+                argnames, &keyFileName, &name1, &name2, &name3, &nrResults,
+                &scanChar))
         return NULL;
 
-    if (strlen(name1) > MXLINELEN - 1)
+    if (scanChar != NULL && PyObject_IsTrue(scanChar)) {
+        isChar = 1;
+    }
+
+    if (strlen(name1) > MXLINELEN)
         return Py_BuildValue("O", result);
     strtolower(name1);
 
@@ -273,7 +280,7 @@ search_name(PyObject *self, PyObject *pArgs, PyObject *pKwds)
         return NULL;
     }
 
-    while (fgets(line, MXLINELEN, keyFile) != NULL) {
+    while (fgets(line, MXLINELEN+1, keyFile) != NULL) {
         /* Split a "origLine|key" line. */
         if ((cp = strrchr(line, FSEP)) != NULL) {
             *cp = '\0';
@@ -290,32 +297,45 @@ search_name(PyObject *self, PyObject *pArgs, PyObject *pKwds)
         strtolower(line);
         strcpy(surname, line);
         hasNS = 0;
-        if ((cp = strrchr(surname, ',')) != NULL && (cp+1)[0] == ' ') {
-            *cp = '\0';
-            hasNS = 1;
-            strcpy(namesurname, cp+2);
-            strcat(namesurname, " ");
-            strcat(namesurname, surname);
+        if (!isChar) {
+            if ((cp = strrchr(surname, ',')) != NULL && (cp+1)[0] == ' ') {
+                *cp = '\0';
+                hasNS = 1;
+                strcpy(namesurname, cp+2);
+                strcat(namesurname, " ");
+                strcat(namesurname, surname);
+            }
+        } else {
+            if ((cp = strrchr(surname, ' ')) != NULL) {
+                    hasNS = 1;
+                    strcpy(namesurname, surname);
+                    strcpy(surname, cp+1);
+                    strcat(surname, "\0");
+            }
         }
-    
+
         ratio = ratcliff(name1, line) + 0.05;
 
         if (hasNS) {
             ratio = MAX(ratio, ratcliff(name1, surname));
-            ratio = MAX(ratio, ratcliff(name1, namesurname));
+            if (!isChar) {
+                ratio = MAX(ratio, ratcliff(name1, namesurname));
+            }
             if (name2 != NULL) {
                 ratio = MAX(ratio, ratcliff(name2, surname));
-                ratio = MAX(ratio, ratcliff(name2, namesurname));
+                if (strcmp(namesurname, "")) {
+                    ratio = MAX(ratio, ratcliff(name2, namesurname));
+                }
             }
         }
 
         if (name3 != NULL && strrchr(origLine, ')') != NULL) {
-            char origLineLower[MXLINELEN];
+            char origLineLower[MXLINELEN+1];
             strcpy(origLineLower, origLine);
             strtolower(origLineLower);
             ratio = MAX(ratio, ratcliff(name3, origLineLower) + 0.1);
-       }
-    
+        }
+
         if (ratio >= RO_THRESHOLD)
             PyList_Append(result, Py_BuildValue("(dis)",
                             ratio, strtol(key, NULL, 16), origLine));
@@ -349,8 +369,8 @@ search_title(PyObject *self, PyObject *pArgs, PyObject *pKwds)
     char *title3 = NULL;
     float ratio;
     FILE *keyFile;
-    char line[MXLINELEN];
-    char origLine[MXLINELEN];
+    char line[MXLINELEN+1];
+    char origLine[MXLINELEN+1];
     char *cp;
     char *key;
     unsigned short hasArt = 0;
@@ -360,7 +380,7 @@ search_title(PyObject *self, PyObject *pArgs, PyObject *pKwds)
     unsigned short linelen = 0;
     unsigned short searchingEpisode = 0;
     unsigned int count = 0;
-    char noArt[MXLINELEN] = "";
+    char noArt[MXLINELEN+1] = "";
     static char *argnames[] = {"keyFile", "title1", "title2", "title3",
                                 "results", NULL};
     PyObject *result = PyList_New(0);
@@ -369,7 +389,7 @@ search_title(PyObject *self, PyObject *pArgs, PyObject *pKwds)
             argnames, &keyFileName, &title1, &title2, &title3, &nrResults))
         return NULL;
 
-    if (strlen(title1) > MXLINELEN - 1)
+    if (strlen(title1) > MXLINELEN)
         return Py_BuildValue("O", result);
 
     strtolower(title1);
@@ -399,13 +419,13 @@ search_title(PyObject *self, PyObject *pArgs, PyObject *pKwds)
                 !strncmp(articlesNoSP[count],
                 &(title1[linelen-artlen]), artlen) &&
                 !strncmp(&(title1[linelen-artlen-2]), ", ", 2)) {
-            /* name1 contains an article. */
+            /* title1 contains an article. */
             hasArt = 1;
             break;
         }
     }
 
-    while (fgets(line, MXLINELEN, keyFile) != NULL) {
+    while (fgets(line, MXLINELEN+1, keyFile) != NULL) {
         /* Split a "origLine|key" line */
         if ((cp = strrchr(line, FSEP)) != NULL) {
             *cp = '\0';
@@ -471,7 +491,7 @@ search_title(PyObject *self, PyObject *pArgs, PyObject *pKwds)
         }
 
         if (title3 != NULL) {
-            char origLineLower[MXLINELEN];
+            char origLineLower[MXLINELEN+1];
             strcpy(origLineLower, origLine);
             strtolower(origLineLower);
             ratio = MAX(ratio, ratcliff(title3, origLineLower) + 0.1);
@@ -493,6 +513,77 @@ search_title(PyObject *self, PyObject *pArgs, PyObject *pKwds)
 }
 
 
+/* Search for the 'name1' in the key file keyFileName, returning at
+ * most nrResults results.
+ */
+static PyObject*
+search_company_name(PyObject *self, PyObject *pArgs, PyObject *pKwds)
+{
+    char *keyFileName = NULL;
+    char *name1 = NULL;
+    float ratio;
+    FILE *keyFile;
+    char line[MXLINELEN+1];
+    char origLine[MXLINELEN+1];
+    char *cp;
+    char *key;
+    short withoutCountry = 1;
+    unsigned int nrResults = 0;
+    static char *argnames[] = {"keyFile", "name1", "results", NULL};
+    float var = 0.0;
+    PyObject *result = PyList_New(0);
+
+    if (!PyArg_ParseTupleAndKeywords(pArgs, pKwds, "ss|i",
+                argnames, &keyFileName, &name1, &nrResults))
+        return NULL;
+
+    if (strlen(name1) > MXLINELEN)
+        return Py_BuildValue("O", result);
+    strtolower(name1);
+
+    if ((keyFile = fopen(keyFileName, "r")) == NULL) {
+        PyErr_SetFromErrno(PyExc_IOError);
+        return NULL;
+    }
+
+    if (line[strlen(line)-1] == ']')
+	    withoutCountry = 0;
+
+    while (fgets(line, MXLINELEN+1, keyFile) != NULL) {
+        /* Split a "origLine|key" line. */
+        if ((cp = strrchr(line, FSEP)) != NULL) {
+            *cp = '\0';
+            key = cp+1;
+            strcpy(origLine, line);
+        } else { continue; }
+	var = 0.0;
+        /* Strip the optional countryCode, if required. */
+        if (withoutCountry && (cp = strrchr(line, '[')) != NULL) {
+            *(cp-1) = '\0';
+	    var = -0.05;
+	}
+
+	strtolower(line);
+
+        ratio = ratcliff(name1, line) + var;
+
+        if (ratio >= RO_THRESHOLD)
+            PyList_Append(result, Py_BuildValue("(dis)",
+                            ratio, strtol(key, NULL, 16), origLine));
+    }
+
+    fclose(keyFile);
+
+    PyObject_CallMethod(result, "sort", NULL);
+    PyObject_CallMethod(result, "reverse", NULL);
+
+    if (nrResults > 0)
+        PySequence_DelSlice(result, nrResults, PySequence_Size(result));
+
+    return Py_BuildValue("O", result);
+}
+
+
 /*========== tv series episodes ==========*/
 /* Return a list of pairs (movieID, "long imdb episode title") with
  * every episode of the given series. */
@@ -507,8 +598,8 @@ get_episodes(PyObject *self, PyObject *pArgs)
     long kfIndex = 0L;
     int i = 0;
     int read_char;
-    char line[MXLINELEN];
-    char series[MXLINELEN];
+    char line[MXLINELEN+1];
+    char series[MXLINELEN+1];
     int seriesLen = 0;
     char *cp;
     char *key;
@@ -517,7 +608,7 @@ get_episodes(PyObject *self, PyObject *pArgs)
 
     if (!PyArg_ParseTuple(pArgs, "lss", &movieID, &indexFileName, &keyFileName))
         return NULL;
-    
+
     if (movieID < 0) {
         PyErr_SetString(PyExc_ValueError, "movieID must be positive.");
         return NULL;
@@ -545,7 +636,7 @@ get_episodes(PyObject *self, PyObject *pArgs)
     }
 
     fseek(keyFile, kfIndex, 0);
-    fgets(series, MXLINELEN, keyFile);
+    fgets(series, MXLINELEN+1, keyFile);
     if ((cp = strrchr(series, FSEP)) != NULL) {
         *cp = '\0';
     }
@@ -554,8 +645,8 @@ get_episodes(PyObject *self, PyObject *pArgs)
     if (series[0] != '"' || series[seriesLen-1] != ')')
         return Py_BuildValue("O", result);
 
-    while (fgets(line, MXLINELEN, keyFile) != NULL) {
-        if (strncmp(line, series, seriesLen)) 
+    while (fgets(line, MXLINELEN+1, keyFile) != NULL) {
+        if (strncmp(line, series, seriesLen))
             break;
 
         if ((cp = strrchr(line, FSEP)) != NULL) {
@@ -595,7 +686,7 @@ pysoundex(PyObject *self, PyObject *pArgs)
 {
     int i, j, n;
     char *s = NULL;
-    char word[MXLINELEN];
+    char word[MXLINELEN+1];
     char soundCode[SOUNDEX_LEN+1];
     char c;
 
@@ -644,6 +735,8 @@ static PyMethodDef cutils_methods[] = {
         METH_KEYWORDS, "Search for a person name."},
     {"search_title", (PyCFunction) search_title,
         METH_KEYWORDS, "Search for a movie title."},
+    {"search_company_name", (PyCFunction) search_company_name,
+        METH_KEYWORDS, "Search for a company name."},
     {"get_episodes", get_episodes,
         METH_VARARGS, "Return a list of episodes of the given series."},
     {"soundex", pysoundex,
