@@ -4,7 +4,7 @@ parser.http.utils module (imdb package).
 This module provides miscellaneous utilities used by
 the imdb.parser.http classes.
 
-Copyright 2004-2007 Davide Alberani <da@erlug.linux.it>
+Copyright 2004-2008 Davide Alberani <da@erlug.linux.it>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -197,13 +197,16 @@ def build_person(txt, personID=None, billingPos=None,
 # To shrink spaces.
 re_spaces = re.compile(r'\s+')
 def build_movie(txt, movieID=None, roleID=None, status=None,
-                accessSystem='http', modFunct=None, _parsingCharacter=False):
+                accessSystem='http', modFunct=None, _parsingCharacter=False,
+                _parsingCompany=False):
     """Given a string as normally seen on the "categorized" page of
     a person on the IMDb's web site, returns a Movie instance."""
-    if not _parsingCharacter:
-        _defSep = '....'
-    else:
+    if _parsingCharacter:
         _defSep = ' Played by '
+    elif _parsingCompany:
+        _defSep = ' ... '
+    else:
+        _defSep = '....'
     title = re_spaces.sub(' ', txt).strip()
     # Split the role/notes from the movie title.
     tsplit = title.split(_defSep, 1)
@@ -283,10 +286,10 @@ def build_movie(txt, movieID=None, roleID=None, status=None,
 class ParserBase(SGMLParser):
     """Base parser to handle HTML data from the IMDb's web server."""
     # The imdbID is a 7-ciphers number.
-    re_imdbID = re.compile(r'(?<=nm|tt|ch)([0-9]{7})\b')
+    re_imdbID = re.compile(r'(?<=nm|tt|ch|co)([0-9]{7})\b')
     re_imdbIDonly = re.compile(r'\b([0-9]{7})\b')
     re_airdate = re.compile(r'(.*)\s*\(season (\d+), episode (\d+)\)', re.I)
-    _re_imdbIDmatch = re.compile(r'(nm|tt|ch)[0-9]{7}\b')
+    _re_imdbIDmatch = re.compile(r'(nm|tt|ch|co)[0-9]{7}/?$')
 
     # It's set when names and titles references must be collected.
     # It can be set to 0 for search parsers.
@@ -364,18 +367,18 @@ class ParserBase(SGMLParser):
         and call the _handle_data method."""
         if self.getRefs:
             if self._inNMRef:
-                self._nameCN += data
+                self._nameCN += data.replace('\n', ' ')
             elif self._inTTRef:
                 if self._inLinkTTRef:
-                    self._titleCN += data
+                    self._titleCN += data.replace('\n', ' ')
                 else:
-                    sdata = data.strip()
+                    sdata = data.strip().replace('\n', ' ')
                     yearK = re_yearKind_index.match(sdata)
                     if yearK and yearK.start() == 0:
                         self._titleCN += ' %s' % sdata[:yearK.end()]
                         self._add_ref('tt')
             elif self._inCHRef:
-                self._characterCN += data
+                self._characterCN += data.replace('\n', ' ')
         self._handle_data(data)
 
     def _handle_data(self, data): pass
